@@ -47,7 +47,7 @@ use List::MoreUtils qw( first_index );
 use List::Util qw( max );
 use Pod::Usage;
 use Pod::Text;
-use ReadonlyX;
+use Readonly;
 use charnames qw(:full);
 
 # package constants
@@ -59,7 +59,7 @@ Readonly::Scalar our $STRINGIFY              => 'Dumper';
 use Amazon::API::Constants qw( :all );
 use Amazon::API::Botocore::Pod qw( pod );
 
-our $VERSION = '2.1.1';  ## no critic (RequireInterpolationOfMetachars)
+our $VERSION = '2.1.3';  ## no critic (RequireInterpolationOfMetachars)
 
 our %BOTO_SERVICES;
 
@@ -766,7 +766,7 @@ In order to create stubs or shapes you must clone the Botocore project
 and provide the path to the project using the -b option or by setting
 the environment variable BOTOCORE_PATH.
 
-  git clone https:/github.com/boto/botocore.git /tmp/botocore
+  git clone https://github.com/boto/botocore.git /tmp/botocore
   export BOTOCORE_PATH=/tmp/botocore
 
 END_OF_CROAKING
@@ -1196,7 +1196,7 @@ our @API_METHODS = qw(
 @actions@
 );
 
-our $VERSION = '2.1.1';
+our $VERSION = '2.1.3';
 
 sub new {
   my ( $class, @options ) = @_;
