@@ -3,7 +3,7 @@ package Amazon::API::Botocore::Shape;
 use strict;
 use warnings;
 
-our $VERSION = '2.1.1';
+our $VERSION = '2.1.3';
 
 use parent qw(Exporter Class::Accessor::Fast);
 
@@ -14,7 +14,7 @@ use Carp;
 use Cwd;
 use Data::Dumper;
 use English qw(-no_match_vars);
-use ReadonlyX;
+use Readonly;
 use Scalar::Util qw( reftype blessed );
 use List::Util qw( any pairs );
 use Log::Log4perl;
