@@ -5,7 +5,7 @@ use warnings;
 
 use parent qw{ Exporter };
 
-use ReadonlyX;
+use Readonly;
 
 our @EXPORT_OK = ();
 
@@ -45,7 +45,7 @@ our %EXPORT_TAGS = (
   ],
 );
 
-our $VERSION = '2.1.1';
+our $VERSION = '2.1.3';
 
 # chars
 Readonly::Scalar our $AMPERSAND          => q{&};
